#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.27
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { getType } from 'js-cool';
import { isNeedUpgrade, upgradeGitmars, queue } from '@gitmars/core';
import { getIsGitProject, getConfig, checkGitStatus } from '@gitmars/git';
import { createArgs } from '@gitmars/utils';
import lang from './common/local.mjs';
import { cmdConfig } from './conf/start.mjs';

const { t } = lang;
const { green, red } = chalk;
const { args, options } = cmdConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const config = getConfig();
program.name("gitm start").usage("<type> <name> [-t --tag <tag>]").description(
  t(
    "Create bugfix task branch, create feature development branch, support framework support branch"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (type, name, opt) => {
  const needUpgrade = await isNeedUpgrade(config.versionControlType);
  needUpgrade && upgradeGitmars();
  const opts = ["bugfix", "feature", "support"];
  const status = checkGitStatus();
  if (!status)
    process.exit(1);
  if (opts.includes(type)) {
    if (opt.tag && type !== "bugfix") {
      sh.echo(
        red(
          t(
            "Specify that only bugfix branch creation is supported when pulling branches from tag"
          )
        )
      );
      process.exit(1);
    }
    if (config.nameValidator) {
      const reg = getType(config.nameValidator) === "regexp" ? config.nameValidator : new RegExp(config.nameValidator);
      if (!reg.test(name)) {
        sh.echo(red(t("Branch name does not conform to specification")));
        process.exit(1);
      }
    }
    name = name.replace(/^\//, "");
    const base = opt.tag ? opt.tag : type === "bugfix" ? config.bugfix : type === "support" ? config.master : config.release;
    const cmd = opt.tag ? ["git fetch", `git checkout -b ${type}/${name} ${base}`] : [
      "git fetch",
      `git checkout ${base}`,
      "git pull",
      `git checkout -b ${type}/${name} ${base}`
    ];
    queue(cmd).then((data) => {
      if (opt.tag && data[1].status === 0 || !opt.tag && data[3].status === 0) {
        sh.echo(
          t(
            'The branch was created successfully and is based on {source}. You have now switched to the {target} branch\nIf you need to test, please run: "{combine}"\nWhen development is complete, remember to run: "{end}"',
            {
              source: String(base),
              target: name,
              combine: green("gitm combine " + type + " " + name),
              end: green("gitm end " + type + " " + name)
            }
          )
        );
      }
    });
  } else {
    sh.echo(red(t("type only allows input") + ": " + JSON.stringify(opts)));
    process.exit(1);
  }
});
program.parse(process.argv);
